@@ -1,7 +1,7 @@
 /* Timing variables for measuring compiler performance.
 
-   Copyright (C) 2000, 2002, 2004, 2009, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 2000, 2002, 2004, 2009-2011 Free Software Foundation,
+   Inc.
 
    Contributed by Alex Samuel <samuel@codesourcery.com>
 
