@@ -1,6 +1,6 @@
 /* Input parser for Bison
 
-   Copyright (C) 2000-2003, 2005-2007, 2009-2010 Free Software
+   Copyright (C) 2000-2003, 2005-2007, 2009-2011 Free Software
    Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -23,6 +23,7 @@
 
 # include "location.h"
 # include "symlist.h"
+# include "named-ref.h"
 
 # include "parse-gram.h"
 
@@ -42,14 +43,17 @@ char const *token_name (int type);
 
 /* From reader.c. */
 void grammar_start_symbol_set (symbol *sym, location loc);
-void grammar_current_rule_begin (symbol *lhs, location loc);
+void grammar_current_rule_begin (symbol *lhs, location loc,
+				 named_ref *lhs_named_ref);
 void grammar_current_rule_end (location loc);
 void grammar_midrule_action (void);
 void grammar_current_rule_prec_set (symbol *precsym, location loc);
 void grammar_current_rule_dprec_set (int dprec, location loc);
 void grammar_current_rule_merge_set (uniqstr name, location loc);
-void grammar_current_rule_symbol_append (symbol *sym, location loc);
-void grammar_current_rule_action_append (const char *action, location loc);
+void grammar_current_rule_symbol_append (symbol *sym, location loc,
+					 named_ref *named_ref);
+void grammar_current_rule_action_append (const char *action, location loc,
+					 named_ref *named_ref);
 void reader (void);
 void free_merger_functions (void);
 
